from Acquisition import Implicit
import Globals
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from OFS.ObjectManager import ObjectManager

def initialize(context):
    context.registerBaseClass(BetterCatalogAware)

class BetterCatalogAware(Implicit,CatalogAware):
    """Mixin base class that fixes three common problems with CatalogAware:
    1. It avoid unnecessary bloat by not unindexing before reindexing
    2. It handles virtual hosting, by inheriting from the underdocumented
       CatalogPathAwareness
    3. It can be used as a base class with ObjectManager.
       Note it must be *before* ObjectManager in the base class list.
    These problems are present in Zope 2.3.2, but may be fixed in a later
    release.
    """
    def manage_afterAdd(self,item,container):
        BetterCatalogAware.inheritedAttribute('manage_afterAdd')(self,item,container)
        if isinstance(self,ObjectManager):
            ObjectManager.manage_afterAdd(self,item,container)
    def manage_afterClone(self,item):
        BetterCatalogAware.inheritedAttribute('manage_afterClone')(self,item)
        if isinstance(self,ObjectManager):
            ObjectManager.manage_afterClone(self,item)
    def manage_beforeDelete(self,item,container):
        BetterCatalogAware.inheritedAttribute('manage_beforeDelete')(self,item,container)
        if isinstance(self,ObjectManager):
            ObjectManager.manage_beforeDelete(self,item,container)
    def reindex_object(self):
        """ Suprisingly useful """
        #self.unindex_object()
        self.index_object()

Globals.default__class_init__(BetterCatalogAware)
